/////////////////////////////////////////////////////////////////////////////
// Project:     wxSQuery
// Purpose:     SQL query tool for the SQLite3 library
// Name:        wxchainlogtextctrl.h
// Author:      Daniel Nell
// Created:     2008/09/26
// Copyright:   (c) Daniel Nell
// Licence:     wxWindows licence
// Modified by:
/////////////////////////////////////////////////////////////////////////////

#ifndef __WXCHAINLOGTEXTCTRL_H
#define __WXCHAINLOGTEXTCTRL_H

/////////////////////////////////////////////////////////////////////////////
//  Class wxChainLogTextCtrl
//
/// \brief A class to split the log output into a wxLogGui and a wxTextCtrl target.
///
/// This is a simple to use class in order to replace the current log target by
/// twice
///     a) a GUI window log with threshold, and
///     b) a text control to log long lists of log output.
class wxChainLogTextCtrl : public wxLogGui
{
  public:
    /// \brief Standard constructor.
    ///
    /// \param poTextctrl    A pointer to a wxTextCtrl object, that is used as an addition
    /// \param nMaxLevel     Refence log level for to suppress messages with higher level,
    ///                      defaults to 2 (errors)
    wxChainLogTextCtrl( wxTextCtrl *poTextctrl, wxLogLevel nMaxLevel= 2 )
    {
        // Save log level.
        m_nMaxLevel= nMaxLevel;

        // Set this object as the new log target.
        wxLog::SetActiveTarget( this );

        // Chain the log to the wxTextCtrl object.
        new wxLogChain( new wxLogTextCtrl( poTextctrl ) );
    }


  protected:
#if wxCHECK_VERSION( 2, 9, 0 )
    /// \brief Internally called function to process the log message.
    ///
    /// Suppress the log output if the given log level is higher then m_nMaxLevel.
    /// So it is possible to get only error messages shown in the normal dialog
    /// window while anything else is logged to the chained wxTextCtrl object.
    ///
    /// \param oLevel        Log level
    /// \param strMessage    Message to log
    /// \param oLogRecInfo   Information about a log record (unit of the log output).
    virtual void DoLogRecord( wxLogLevel nLevel,
                              const wxString& strMessage,
                              const wxLogRecordInfo& oLogRecInfo )
    {
        // Show message only if the log level is lower or equal to the reference value.
        if( nLevel <= m_nMaxLevel )
        {
            // Show message using parent function.
            wxLogGui::DoLogRecord( nLevel, strMessage, oLogRecInfo );
        }
    }
#else
    /// \brief Internally called function to process the log message.
    ///
    /// Suppress the log output if the given log level is higher then m_nMaxLevel.
    /// So it is possible to get only error messages shown in the normal dialog
    /// window while anything else is logged to the chained wxTextCtrl object.
    ///
    /// \param oLevel        Log level
    /// \param pszMessage    Message to log
    /// \param nTimestamp    Timestamp of the message
    virtual void DoLog( wxLogLevel nLevel, const wxChar *pszMessage, time_t nTimestamp )
    {
        // Show message only if the log level is lower or equal to the reference value.
        if( nLevel <= m_nMaxLevel )
        {
            // Show message using parent function.
            //
            // Note: BASECLASS::DoLog() is protected. Therefore we cannot remember
            //       a pointer to the old log target and send the message but we have
            //       to subclass from wxLogGui and use its DoLog() function.
            wxLogGui::DoLog( nLevel, pszMessage, nTimestamp );
        }
    }
#endif // wxCHECK_VERSION

  protected:
    // Member variable(s).
    wxLogLevel m_nMaxLevel;     /// Log level to propagate messages.
};


#endif // __WXCHAINLOGTEXTCTRL_H
