/***************************************************************
 * Name:      wxiscanframe.h
 * Purpose:   Code for application frame (the main) window
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2008-05-04
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXISCANFRAME_H
#define WXISCANFRAME_H

//(*Headers(wxIScanFrame)
#include <wx/toolbar.h>
#include <wx/menu.h>
#include <wx/scrolwin.h>
#include <wx/splitter.h>
#include <wx/statusbr.h>
#include <wx/frame.h>
#include <wx/textctrl.h>
//*)

// Additional wxWidgets header files
#include <wx/docview.h>
#include <wx/process.h>

// Private header files.
#include "wxextfiledroptarget.h"
#include "wximagepanel.h"
#include "wxiviewcanvas.h"
#include "wxiviewpaint.h"
#include "wxscan.h"
#if __WXPDFDOC__
#   include "wxiscanhocr2pdf.h"
#endif // __WXPDFDOC__

/////////////////////////////////////////////////////////////////////////////
//  Constants
//
#define wxISCAN_SCRIPTMENUPOS           5

/////////////////////////////////////////////////////////////////////////////
// IDs
//
#define wxISCAN_PROCESS                 1


//////////////////////////////////////////////////////////
// Predefined classes
//
class wxExtProcess;


//////////////////////////////////////////////////////////
// Class wxIScanFrame
//
/// \brief The frame window class.
///
/// wxIScanFrame represents the scanning application's main
/// window and handles most of the events (e. g. menu, timer
/// etc.).
///
/// Note:
///         In addition to wxFrame wxIScanFrame inherits the
///         class wxIViewPaintBase which is an interface to
///         basic printing and print previewing of an image.
class wxIScanFrame: public wxFrame, public wxIViewPaintBase
{
    public:
        enum
        {
            wxCANVASFULLSIZE,                                       ///< Show image full size (without any zooming).
            wxCANVASFITWIDTH,                                       ///< Zoom image to fit the window width.
            wxCANVASFITWINDOW,                                      ///< Zoom image to fit the window size.
            wxCANVASCUSTOMSIZE                                      ///< Zoom image to an explicit given zoom factor (m_nScaleFactor).
        };

    public:
        /// \brief Standard constructor.
        ///
        /// \param poParent                  a pointer to the parent window
        /// \param nId                       the window id
        wxIScanFrame( wxWindow* poParent, wxWindowID nId= wxID_ANY );

        /// \brief Virtual destructor.
        virtual ~wxIScanFrame();

        virtual bool CanClose( bool bCanVeto= true );               ///< Handle window closing requests.
        virtual bool OpenImage( const wxString &strFileName, bool bSetFileName= true );  ///< Open an image using the given name.
        virtual bool OnDocumentPageChanged();                       ///< (Re)loads the (zero based) page number given in m_nPage.
        virtual void UpdateBitmap();                                ///< Update the canvas bitmap.
        virtual wxImage GetImageSelection();                        ///< Get the selected sub image.
        virtual bool ScanImage();                                   ///< Get an image from scanner.
#ifdef __WXSCANSANE__
        virtual void ScanForScanners( bool bAsync= false );         ///< Scan for existing SANE scanners an fill the object in the tool bar.
        virtual void ScanForScannersThreadEntry();                  ///< Create a new scanner object and scan for scanning devices.
        virtual void ScanForScannersThreadExit();                   ///< Fill the wxChoice object for scanners with a list of available scanning devices.
#endif // __WXSCANSANE__
        virtual void SetProfile( int nProfileIndex );               ///< Select a profile (by number).
        virtual void ApplyProfile();                                ///< Apply the selected profile to the image.

        /// \brief Enhance the optical contrast by replacing white by yellow.
        virtual void EnhenceOpticalContrast()
        {
            m_oImage.Replace( 255, 255, 255, 255, 255, 0 );
            UpdateBitmap();
        }

        /// \brief Call an external OCR software and return the
        ///        filename of the resulting text file.
        ///
        /// \param strTempFileName           the file name of the image that should be read by OCR().
        /// \param bUseHocr                  flag if hOCR (as html) output should be done, defaults to false.
        virtual wxString OCR( const wxString &strTempFileName, bool bUseHocr= false );

        /// \brief Do OCR and read out the returned text.
        ///
        /// \param bAsync                    start the read out asynchronously, defaults to true.
        virtual bool ReadOut( bool bAsync= true );

        /// \brief Fork an external process.
        ///
        /// \param strCommand                the command line to be executed.
        /// \param bAsync                    an (optional) flag which decides if the new process
        ///                                  is to be executed synchronous (default) or asynchronous.
        /// \param strTempTextFileName       an (optional) file name of a file that should be deleted
        ///                                  at the end of an asynchronous running process.
        virtual bool Execute( const wxString &strCommand, bool bAsync= false,
                              const wxString &strTempTextFileName= wxEmptyString );

#if __WXPDFDOC__
        /// \brief Create (and initialize) a new PDF document.
        virtual bool CreateNewPdfDocument();

        /// \brief Add a new image/OCR text page to the existing PDF document.
        virtual bool AddPdfPage();

        /// \brief Check if the PDF dirty flag is set to TRUE.
        virtual bool IsPdfDirty()
        {
            return m_bPdfDirty;
        }
#endif // __WXPDFDOC__

        /// \brief State if the image is in portrait direction.
        virtual bool IsPortrait()
        {
            return ( m_oImage.GetHeight() > m_oImage.GetWidth() );
        }

        /// \brief Reset the filename to "unknown.jpg" etc.
        virtual void ResetFileName();

        /// \brief Write a sub processes stdout and stderr output to the program's log.
        virtual void LogProcessOutput( wxProcess *poProcess );

        /// \brief Update the page number info in the statusbar and in the toolbar.
        virtual void UpdatePageNumber();

        /// \brief Toggle full screen mode.
        virtual void ToggleFullScreen()
        {
            ShowFullScreen( !IsFullScreen() );
        }

        /// \brief Restore program settings from config file.
        virtual void RestoreSettings();

        /// \brief Restore sash settings from config file.
        virtual void RestoreSashSettings();

        /// \brief Save program settings in config file.
        virtual void SaveSettings();

        /// \brief Enable or disable controls according to the actual progam state.
        virtual void EnableControls();

        /// \brief Locks/Unlocks the user interface.
        ///
        /// \param bLock                    if true locks otherwise unlocks the user interface.
        virtual void LockUI( bool bLock= true )
        {
            m_bUILocked= bLock;
            EnableControls();
        }

        /// \brief Unlocks the user interface.
        virtual void UnlockUI()
        {
            LockUI( false );
        }

#if __WXLUASCRIPT__
        /// \brief Start a Lua script from a give file.
        ///
        /// \param strScriptStartupFunction  startup function of the Lua script
        /// \param strScriptFileName         file name of the text file containing the Lua script
        ///                                  (defaults to the empty string which means, that no
        ///                                  script is to be loaded before running the start up
        ///                                  function).
        virtual void StartScript( const wxString &strScriptStartupFunction,
                                  wxString strScriptFileName= wxEmptyString );
#endif // __WXLUASCRIPT__

#if __WXPDFDOC__
    private:
        // Private helper functions.
        void AddPdfPageTraverseXmlNodes( wxXmlNode *poNode );       ///< DOM tree traverse helper function for AddPdfPage().
#endif // __WXPDFDOC__

    public:
        // Printer handling (introduced by base class wxIViewPaintBase)
        virtual void AdjustOrientation(){ m_oPrintData.SetOrientation( IsPortrait() ? wxPORTRAIT : wxLANDSCAPE ); } ///< ...
        virtual bool GetPageSizeMM( int *pnWidth, int *pnHeight );  ///< Calculates the image extents in mm while printing or print previewing.
        virtual int  GetMaxPage(){ return m_nImageCount; }          ///< Get the (zero based) page number of the last page.
        virtual bool GotoPage( int nPage );                         ///< Go to a given printer page.
        virtual bool Paint( wxDC& oDc );                            ///< Paints the image while printing or print previewing.

    private:
        // Member variables
        bool m_bUILocked;                                           ///< States if the GUI should be locked by wxIScanFrame::EnableControls().
        wxImage m_oImage;                                           ///< Image object.
        wxImage m_oPrintImage;                                      ///< Image object for printing.
        wxString m_strImageFileName;                                ///< Image file name.
        int m_nPage;                                                ///< Page number.
        int m_nImageCount;                                          ///< The number of images in an image file.
        int m_nResolution;                                          ///< The (fictive) resolution of an image in dpi.
        wxScan *m_poScanner;                                        ///< Object representing the scanner.
#ifdef __WXSCANSANE__
        wxCriticalSection m_oCriticalSectionScanner;                ///< critical section to protect m_poScanner.
#endif // __WXSCANSANE__

        // Options
        int m_nFit;                                                 ///< Fit bitmap size according to its value.
        double m_nScaleFactor;                                      ///< Bitmap scalation factor.
        double m_nZoomFactor;                                       ///< Zoom factor by which the bitmap should be scaled.
        bool m_bAutoApplyProfile;                                   ///< Apply a profile automatically after scanning.
        bool m_bAutoApplyProfileOnMultiPage;                        ///< Apply a profile automatically after loading a page from a multi page file.
        int m_nProfileIndex;                                        ///< Index number of the current image profile.
        wxString m_strBitmapMimeType;                               ///< Image mimetype for any export but PDF export.
#if __WXPDFDOC__
        wxString m_strPdfBitmapMimeType;                            ///< Image mimetype for PDF export.
#endif // __WXPDFDOC__
#if __WXLUASCRIPT__
        bool m_bLua;                                                ///< Enable Lua scripting.
        int  m_nLuaCount;                                           ///< Number of Lua scripts.
#endif // __WXLUASCRIPT__
        bool m_bOCR;                                                ///< Enable OCR.
        wxString m_strOcrOutputExtension;                           ///< The output extension of the via OCR extracted text file.
        wxString m_strOcrOutputExtensionHocr;                       ///< The output extension of the via OCR extracted hocr (html) file.
        bool m_bCmdConvertEnable;                                   ///< ...
        wxString m_strCmdConvert;                                   ///< ...
        bool m_bCmdOcrEnable;                                       ///< ...
        wxString m_strCmdOcr;                                       ///< ...
        bool m_bCmdCleanUpEnable;                                   ///< ...
        wxString m_strCmdCleanUp;                                   ///< ...
        bool m_bHocrEnable;                                         ///< ...
        wxString m_strHocrClass;                                    ///< ...
        bool m_bSmartHocrEnable;                                    ///< ...
        wxString m_strCmdOcrUseHocr;                                ///< ...
        bool m_bCmdHtml2XhtmlEnable;                                ///< ...
        wxString m_strCmdHtml2Xhtml;                                ///< ...
        bool m_bCmdHocrCleanUpEnable;                               ///< ...
        wxString m_strCmdHocrCleanUp;                               ///< ...
        bool m_bCmdTtsEnable;                                       ///< ...
        wxString m_strCmdTts;                                       ///< ...
        bool m_bCmdEditEnable;                                      ///< ...
        wxString m_strCmdEdit;                                      ///< ...

        // File history
        wxFileHistory m_oFileHistory;                               ///< History of last opened files

        // Printing support.
        wxPrintData m_oPrintData;                                   ///< Holds related to printers and printer device contexts.

#if __WXPDFDOC__
        // PDF support
        wxPdfDocument *m_poPdfDoc;                                  ///< PDF document.
        int m_nPdfPages;                                            ///< Number of PDF pages in memory.
        bool m_bPdfDirty;                                           ///< States if the PDF file is dirty (and should be saved).
#endif // __WXPDFDOC__

#if __WXLUASCRIPT__
        bool bLuaReturnFromEventHandler;                            ///< Lua return value of the event handlers.
#endif // __WXLUASCRIPT__

        // Managed process.
        wxExtProcess *m_poRunningProcess;                           ///< A concurrent running process (equals to NULL if there isn't any).

    private:
        //(*Handlers(wxIScanFrame)
        void OnFileQuit(wxCommandEvent& event);
        void OnFileScanImage(wxCommandEvent& event);
        void OnFileOpen(wxCommandEvent& event);
        void OnFileSaveAs(wxCommandEvent& event);
        void OnFilePrint( wxCommandEvent &oEvent );
        void OnFilePrintPreview( wxCommandEvent &oEvent );
        void OnFilePrintSetup( wxCommandEvent &oEvent );
        void OnEditCopy(wxCommandEvent& event);
        void OnPdfBeginPdf(wxCommandEvent& event);
        void OnDocumentCopyText(wxCommandEvent& event);
        void OnDocumentReadText(wxCommandEvent& event);
        void OnDocumentFirstPage(wxCommandEvent& event);
        void OnDocumentPreviousPage(wxCommandEvent& event);
        void OnDocumentNextPage(wxCommandEvent& event);
        void OnDocumentLastPage(wxCommandEvent& event);
        void OnDocumentRotate90Left(wxCommandEvent& event);
        void OnDocumentRotate90Right(wxCommandEvent& event);
        void OnDocumentApplyProfile(wxCommandEvent& event);
        void OnPdfAddPage(wxCommandEvent& event);
        void OnPdfSavePdf(wxCommandEvent& event);
        void OnHelpAbout(wxCommandEvent& event);
        void OnCmdStop(wxCommandEvent& event);
        void OnClose(wxCloseEvent& event);
        void OnEditPaste(wxCommandEvent& event);
        void OnEditSettings(wxCommandEvent& event);
        void OnEditSettingsFile(wxCommandEvent& event);
        void OnDocumentCrop(wxCommandEvent& event);
        void OnEditClearHistory(wxCommandEvent& event);
        void OnEditClearLog(wxCommandEvent& event);
        void OnDocumentZoom(wxCommandEvent& event);
        void OnViewFit(wxCommandEvent& event);
        void OnViewFullScreen(wxCommandEvent& event);
        void OnEditScanScanners(wxCommandEvent& event);
        void OnViewCloseDebugWindow(wxCommandEvent& event);
        void OnViewClosePreviewPanel(wxCommandEvent& event);
        //*)
        void OnDocumentProfilesChooseProfile( wxCommandEvent& oEvent ); ///< Handle menu command "Document|Profile|[PROFILE #X]".
#if __WXLUASCRIPT__
        void OnScriptStartScript( wxCommandEvent& oEvent );         ///< Handle menu command "Scripts|[SCRIPT NAME]".
#endif // __WXLUASCRIPT__
        void OnMRUFiles( wxCommandEvent& oEvent );                  ///< Handle menu command "File|[MOST RECENTLY USED FILE #X]".
        void OnFileDrop( wxExtFileDropEvent& oEvent );              ///< Handle file dropping.
        void OnCanvasSize( wxIViewCanvasSizeEvent& oEvent );        ///< Handle bitmap resizing introduced by canvas resizing.
        void OnCanvasSelected( wxIViewCanvasSelectedEvent& oEvent ); ///< Handle canvas selection events.
        void OnProcessTerminated( wxProcessEvent& oEvent );         ///< Handle process termination.
        void OnTextCtrlPageProcessEnter( wxCommandEvent& oEvent );  ///< Handle page number editor field events.

#ifdef __WXSCANSANE__
        /// \brief Handle the exit of another thread.
        ///
        /// \param oEvent       ... (not yet used at all).
        void OnThreadExit( wxCommandEvent& oEvent )
        {
            ScanForScannersThreadExit();
        }
#endif // __WXSCANSANE__

        //(*Identifiers(wxIScanFrame)
        static const long ID_CANVAS;
        static const long ID_PAGES;
        static const long ID_VSPLITTERWINDOW;
        static const long ID_LOGTEXTCTRL;
        static const long ID_HSPLITTERWINDOW;
        static const long ID_FILESCANIMAGE;
        static const long ID_FILEOPEN;
        static const long ID_FILESAVEAS;
        static const long ID_FILEPRINT;
        static const long ID_FILEPRINTPREVIEW;
        static const long ID_FILEPRINTSETUP;
        static const long ID_FILEQUIT;
        static const long ID_EDITCOPY;
        static const long ID_EDITPASTE;
        static const long ID_EDITSETTINGS;
        static const long ID_EDITSETTINGSFILE;
        static const long ID_EDITSCANSCANNERS;
        static const long ID_EDITCLEARLOG;
        static const long ID_EDITCLEARHISTORY;
        static const long ID_VIEWFULLSCREEN;
        static const long ID_VIEWFITWIDTH;
        static const long ID_VIEWFITWINDOW;
        static const long ID_VIEWFULLSIZE;
        static const long ID_VIEWCUSTOMSIZE;
        static const long ID_VIEWZOOMIN;
        static const long ID_VIEWZOOMOUT;
        static const long ID_VIEWZOOMSELECTION;
        static const long ID_VIEWCLOSEDEBUGWINDOW;
        static const long ID_VIEWCLOSEPREVIEWPANEL;
        static const long ID_DOCUMENTCOPYTEXT;
        static const long ID_DOCUMENTREADTEXT;
        static const long ID_CMDSTOP;
        static const long ID_DOCUMENTFIRSTPAGE;
        static const long ID_DOCUMENTPREVIOUSPAGE;
        static const long ID_DOCUMENTNEXTPAGE;
        static const long ID_DOCUMENTLASTPAGE;
        static const long ID_DOCUMENTROTATE90LEFT;
        static const long ID_DOCUMENTROTATE90RIGHT;
        static const long ID_DOCUMENTCROP;
        static const long ID_DOCUMENTAPPLYPROFILE;
        static const long ID_MENUDOCUMENTPROFILES;
        static const long ID_PDFBEGINPDF;
        static const long ID_PDFADDPAGE;
        static const long ID_PDFSAVEASPDF;
        static const long ID_HELPABOUT;
        static const long ID_FILESCANIMAGE_TB;
        static const long ID_FILEOPEN_TB;
        static const long ID_FILESAVEAS_TB;
        static const long ID_FILEPRINT_TB;
        static const long ID_PDFBEGINPDF_TB;
        static const long ID_PDFADDPAGE_TB;
        static const long ID_PDFSAVEASPDF_TB;
        static const long ID_DOCUMENTFIRSTPAGE_TB;
        static const long ID_DOCUMENTPREVIOUSPAGE_TB;
        static const long ID_DOCUMENTNEXTPAGE_TB;
        static const long ID_DOCUMENTLASTPAGE_TB;
        static const long ID_DOCUMENTREADTEXT_TB;
        static const long ID_CMDSTOP_TB;
        //*)
        static const long ID_DOCUMENTREADTEXTSYNC;                  ///< ...
        static const long ID_DOCUMENTPROFILE0;                      ///< ...
        static const long ID_TEXTCTRLPAGE;                          ///< ...
#ifdef __WXSCANSANE__
        static const long ID_CHOICESCANNER;                         ///< ...
#endif // __WXSCANSANE__

        //(*Declarations(wxIScanFrame)
        wxIViewCanvas* m_poCanvas;
        wxToolBarToolBase* ToolBarItem5;
        wxToolBarToolBase* ToolBarItem11;
        wxSplitterWindow* m_poVSplitterWindow;
        wxToolBarToolBase* ToolBarItem6;
        wxImagePanel* m_poPanelPages;
        wxToolBarToolBase* ToolBarItem12;
        wxToolBarToolBase* ToolBarItem7;
        wxToolBarToolBase* ToolBarItem2;
        wxToolBarToolBase* ToolBarItem13;
        wxToolBarToolBase* ToolBarItem9;
        wxToolBarToolBase* ToolBarItem10;
        wxToolBarToolBase* ToolBarItem4;
        wxMenu* poMenuEdit;
        wxToolBarToolBase* ToolBarItem1;
        wxToolBarToolBase* ToolBarItem3;
        wxSplitterWindow* m_poHSplitterWindow;
        wxTextCtrl* m_poLogTextCtrl;
        wxToolBarToolBase* ToolBarItem8;
        //*)
        wxTextCtrl *m_poTextCtrlPage;                               ///< A pointer to the page number editor
        wxTextValidator m_oTextValidatorTextCtrlPage;               ///< The validator to the page number editor.
#ifdef __WXSCANSANE__
        wxChoice *m_poChoiceScanner;                                ///< A pointer to the scanner choice dropdown box.
#endif // __WXSCANSANE__

        // Friend classes.
#if __WXLUASCRIPT__
        friend class wxIScanLuaScript;
#endif // __WXLUASCRIPT__

        // Event table.
        DECLARE_EVENT_TABLE()
};


#ifdef __WXSCANSANE__
//////////////////////////////////////////////////////////
//  class wxIScanImageThread
//
/// \brief ...
///
/// ...
class wxIScanFrameScanForScannersThread : public wxExtThread
{
  public:
    /// \brief Standard constructor
    ///
    /// \param poEvtHandler             A pointer to the event handler that owns the thread.
    /// \param nEventInt                The ID of the corresponding image (in the hash map).
    wxIScanFrameScanForScannersThread( wxEvtHandler *poEvtHandler )
      : wxExtThread( poEvtHandler )
    {
    }

    /// \brief Thread execution starts here.
    virtual ExitCode Entry()
    {
        // Run the preview image creation.
        ( (wxIScanFrame *)m_poEvtHandler )->ScanForScannersThreadEntry();

        // Return with exit code 0 (success).
        return 0;
    }
};
#endif // __WXSCANSANE__

#endif // WXISCANFRAME_H
