/***************************************************************
 * Name:      wxiscanhocr2pdf.cpp
 * Purpose:   ...
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2012-02-03
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

// Include precompiled headers.
#include "wx_pch.h"


#if __WXPDFDOC__
// wxWidgets headers
#   include <wx/pdfdoc.h>
#   include <wx/tokenzr.h>
#   include <wx/xml/xml.h>

// Private headers
#   include "wxiscanhocr2pdf.h"



//////////////////////////////////////////////////////////
// Class wxIScanFrameHocr2Pdf
//
// Standard constructor.
//
wxIScanHocr2Pdf::wxIScanHocr2Pdf( wxXmlDocument *poXmlDoc, wxPdfDocument *poPdfDoc,
                                 int nResolution, const wxString &strHocrClassFilter )
 : m_strHocrClassFilter( strHocrClassFilter ),
   m_poXmlDoc( poXmlDoc ),
   m_poPdfDoc( poPdfDoc ),
   m_nResolution( nResolution )
{
    // Initialization. (Nothing to do, yet.)
}

// Traverse the DOM tree by calling TraverseXmlNodes()
// and flushing outstanding operations in the end.
//
void wxIScanHocr2Pdf::Run()
{
    TraverseXmlNodes( m_poXmlDoc->GetRoot() );
    Flush2Pdf();
}

// Traverse recursively through the DOM tree beginning with the given node.
//
void wxIScanHocr2Pdf::TraverseXmlNodes( wxXmlNode *poNode )
{
    // If this is an XML tag containing a 'title' attribute
    // beginning with 'bbox' extract the bounding box and
    // the content (the text) and print it on the PDF page
    // using the (lower left corner of the) bounding box.
    if( poNode->GetType() == wxXML_ELEMENT_NODE )
    {
        wxString strAttrClass= poNode->GetAttribute( wxT( "class" ), wxEmptyString );
        wxString strAttrTitle= poNode->GetAttribute( wxT( "title" ), wxEmptyString );

        if(    ( strAttrClass.IsEmpty() || !strAttrClass.Cmp( m_strHocrClassFilter ) )
            && strAttrTitle.StartsWith( wxT( "bbox" ) ) )
        {
            // Parse string, ...
            wxArrayString astrTokens= wxStringTokenize( strAttrTitle );

            // ... get the coordinates of the bounding box, and ...
            long x, y;

            astrTokens[1].ToLong( &x );
            astrTokens[4].ToLong( &y );

            // ... "print" the text on the PDF page.
            Print2Pdf( (double)x / (double)m_nResolution * 25.4,
                       (double)y / (double)m_nResolution * 25.4,
                       GetNodeContent( poNode ) );
        }
    }
    else if( poNode->IsWhitespaceOnly() )
    {
        // Flush eventually delayed output.
        Flush2Pdf();
    }

    // Do the same for all children of this XML node
    // (so doing a depth first search).
    for( wxXmlNode *poChildNode= poNode->GetChildren(); poChildNode; poChildNode= poChildNode->GetNext() )
    {
        TraverseXmlNodes( poChildNode );
    }
}

// Get all text content from all levels below this node.
//
wxString wxIScanHocr2Pdf::GetNodeContent( wxXmlNode *poNode )
{
    // Get the current node's text content...
    wxString strContent= poNode->GetNodeContent();

    // ... and concatenate with the child node's text content.
    for( wxXmlNode *poIteratorNode= poNode->GetChildren(); poIteratorNode; poIteratorNode= poIteratorNode->GetNext() )
    {
        strContent += GetNodeContent( poIteratorNode );
    }
    return strContent;
}

// Print the given text at the given coordinates.
//
void wxIScanHocr2Pdf::Print2Pdf( double x, double y, const wxString& strText )
{
    m_poPdfDoc->Text( x, y, strText );
}


//////////////////////////////////////////////////////////
// Class wxIScanSmartHocr2Pdf
//
// Standard constructor.
//
wxIScanSmartHocr2Pdf::wxIScanSmartHocr2Pdf( wxXmlDocument *poXmlDoc, wxPdfDocument *poPdfDoc,
                                            int nResolution, const wxString &strHocrClassFilter )
  : wxIScanHocr2Pdf( poXmlDoc, poPdfDoc, nResolution, strHocrClassFilter )
{
    // Clear the collection buffer.
    m_strText.Empty();

    // Reset ...
    m_y= -1.0;
}

// "Print" the given text at the given coordinates.
//
// Note: This function only collects (delays) text until a
//       flush is made using Flush2Pdf().
//
void wxIScanSmartHocr2Pdf::Print2Pdf( double x, double y, const wxString& strText )
{
    // ...
    if( m_y < 0.0 )
    {
        m_x= x;
        m_y= y;
    }

    // Add the new text to the already collected text.
    m_strText += strText;
}

// Flush outstanding print-to-PDF-commands.
//
// Note: This function flushes the text collected (delayed)
//       by Print2Pdf().
//
void wxIScanSmartHocr2Pdf::Flush2Pdf()
{
    if( m_y >= 0.0 )
    {
        // "Print" the collected text.
        m_poPdfDoc->Text( m_x, m_y, m_strText );

        // Empty the collection buffer.
        m_strText.Empty();

        // Reset...
        m_y= -1.0;
    }
}
#endif // __WXPDFDOC__
