/***************************************************************
 * Name:      wxiscanhocr2pdf.h
 * Purpose:   ...
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2012-02-03
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXISCANHOCR2PDF_H
#define WXISCANHOCR2PDF_H


//////////////////////////////////////////////////////////
// Predefined classes
//
#if __WXPDFDOC__
class wxPdfDocument;
class wxXmlDocument;
class wxXmlNode;


//////////////////////////////////////////////////////////
// Class wxIScanFrameHocr2Pdf
//
/// \brief DOM tree traverser class for
///        wxIScanFrame::AddPdfPage().
///
/// This is a helper class that traverses a hOCR DOM tree
/// and "prints" the text of a hOCR XML file hidden behind
/// the corresponding position on the image.
///
/// NOTE:
///
/// 1) This is some sort of a "private" class to wxIScanFrame
///    and should not be used outside wxIScanFrame::AddPdfPage().
///
/// 2) There is no validity check done on poXmlDoc, poPdfDoc
///    and nResolution. That is it is assumed that all
///    parameters of the constructor are valid.
class wxIScanHocr2Pdf
{
  public:
    /// \brief Standard constructor.
    ///
    /// \param poXmlDoc             the (valid!) pointer to the XML DOM tree
    /// \param poPdfDoc             the (valid!) pointer to the PDF document
    /// \param nResolution          the (virtual) resolution of the image
    /// \param strHocrClassFilter   the class to use for hOCR information (e. g. whole lines or words)
    wxIScanHocr2Pdf( wxXmlDocument *poXmlDoc,
                     wxPdfDocument *poPdfDoc,
                     int nResolution,
                     const wxString &strHocrClassFilter= wxT( "ocr_line" ) );

    /// \brief Virtual destructor;
    ~wxIScanHocr2Pdf(){}

    /// \brief Traverse the DOM tree by calling TraverseXmlNodes()
    ///        and flushing outstanding operations in the end.
    virtual void Run();

  protected:

    /// \brief Traverse recursively through the DOM tree
    ///        beginning with the given node.
    ///
    /// \param poNode      pointer to start node in the DOM tree.
    virtual void TraverseXmlNodes( wxXmlNode *poNode );

    /// \brief Get all text content from all levels below this node.
    ///
    /// \param poNode      pointer to current node in the DOM tree.
    virtual wxString GetNodeContent( wxXmlNode *poNode );

    /// \brief Print the given text at the given coordinates.
    ///
    /// \param x            abscissa of the origin
    /// \param y            ordinate of the origin
    /// \param strText      text to print
    ///
    /// NOTE:
    ///        If you want to change the behaviour of the text placement
    ///        you should override this function.
    virtual void Print2Pdf( double x, double y, const wxString& strText );

    /// \brief Flush outstanding print-to-PDF-commands.
    ///
    /// NOTE:
    ///        This function does nothing, but can be overriden.
    virtual void Flush2Pdf(){}

  public:
    wxString m_strHocrClassFilter;      ///< Filter for the hOCR 'class' attribute.

  protected:
    wxXmlDocument *m_poXmlDoc;          ///< The pointer to the XML document.
    wxPdfDocument *m_poPdfDoc;          ///< The pointer to the PDF document.
    int m_nResolution;                  ///< The (fictive) resolution of an image in dpi.
};


//////////////////////////////////////////////////////////
// Class wxIScanSmartHocr2Pdf
//
/// \brief Traverse recursively through the DOM tree
///        beginning with the given node.
///
/// This class extends wxIScanHocr2Pdf by "smart" glueing
/// the letters of one word together. This makes it easer
/// to search a PDF document for words (instead of single)
/// letters).
///
/// NOTE:
///
/// 1) This is some sort of a "private" class to wxIScanFrame
///    and should not be used outside wxIScanFrame::AddPdfPage().
///
/// 2) There is no validity check done on poXmlDoc, poPdfDoc
///    and nResolution. That is it is assumed that all
///    parameters of the constructor are valid.
class wxIScanSmartHocr2Pdf: public wxIScanHocr2Pdf
{
  public:
    /// \brief DOM tree traverser class for wxIScanFrame::AddPdfPage()
    ///        based on wxIScanFrameHocr2Pdf class to achieve a
    ///        "smarter" word based PDF output hidden behind the image.
    ///
    /// \param poXmlDoc             the (valid!) pointer to the XML DOM tree
    /// \param poPdfDoc             the (valid!) pointer to the PDF document
    /// \param nResolution          the (virtual) resolution of the image
    /// \param strHocrClassFilter   the class to use for hOCR information (e. g. whole lines or words)
    wxIScanSmartHocr2Pdf( wxXmlDocument *poXmlDoc,
                          wxPdfDocument *poPdfDoc,
                          int nResolution,
                          const wxString &strHocrClassFilter= wxT( "ocr_line" ) );

  protected:
    /// \brief Print the given text at the given coordinates.
    ///
    /// \param x            abscissa of the origin
    /// \param y            ordinate of the origin
    /// \param strText      text to print
    ///
    /// NOTE:
    ///        This function only collects text until Flush2Pdf()
    ///        is called.
    virtual void Print2Pdf( double x, double y, const wxString& strText );

    /// \brief Flush outstanding print-to-PDF-commands.
    ///
    /// NOTE:
    ///        This function does the real printing initialized by
    ///        (maybe multiple) calls to Print2Pdf().
    virtual void Flush2Pdf();

  protected:
    double m_x, m_y;                    ///< Save printing coordinates.
    wxString m_strText;                 ///< Save outstanding text.
};
#endif // __WXPDFDOC__

#endif // WXISCANHOCR2PDF_H
