/////////////////////////////////////////////////////////////////////////////
// Project:     wxIView
// Purpose:     Complex wxWidgets sample
// Name:        wxoptionsdialog.h
// Author:      Daniel Nell
// Created:     2006/02/16
// Copyright:   (c) Daniel Nell
// Licence:     wxWindows licence
// Modified by:
/////////////////////////////////////////////////////////////////////////////

#ifndef __WXOPTIONSDIALOG_H
#define __WXOPTIONSDIALOG_H


/////////////////////////////////////////////////////////////////////////////
//  Constants
//
#define WXOPTIONSDIALOGMAXOPTION   100
#define WXOPTIONSDIALOGIDOPTION    31101


/////////////////////////////////////////////////////////////////////////////
//  Forward declarations
//
class wxBoxSizer;
class wxCheckBox;
class wxStaticLine;
class wxBookCtrlBase;


/////////////////////////////////////////////////////////////////////////////
//  class wxOptionBase
//
/// \brief The (abstract) option for an options dialog.
///
/// This class represents an (abstract) option (more or less an inteface)
/// used by inheriting classes in an options dialog.
class wxOptionBase
{
  public:
    /// Default constructor.
    wxOptionBase(){}

    /// Virtual destructor.
    virtual ~wxOptionBase(){};

  public:
    // Abstract member functions
    virtual bool AddToSizer( wxBoxSizer *poSizer ) = 0;    ///< ...
    virtual void Setup( wxWindow *poWindow, int nId ) = 0; ///< ...
    virtual void StoreValue() = 0;                         ///< In implementation: Store the value in the connected variables.
};


/////////////////////////////////////////////////////////////////////////////
//  class wxOptionStaticLine
//
/// \brief The static line pseudo option for an options dialog.
///
/// This class represents a static line object used in an option
/// dialog.
class wxOptionStaticLine : public wxOptionBase
{
  public:
    /// \brief Default constructor.
    wxOptionStaticLine();

  public:
    // Member functions
    virtual bool AddToSizer( wxBoxSizer *poSizer );        ///< ...
    virtual void Setup( wxWindow *poWindow, int nId );     ///< ... (Nothing to do.)
    virtual void StoreValue(){}                            ///< Store the value in the connected variables. (Nothing to do.)

  protected:
    // Member variables.
    wxStaticLine *m_poStaticLine;
};


/////////////////////////////////////////////////////////////////////////////
//  class wxOptionCheckBox
//
/// \brief The boolean option for an options dialog.
///
/// This class represents a boolean (checkbox) option used in an
/// options dialog.
class wxOptionCheckBox : public wxOptionBase
{
  public:
    /// \brief Default constructor.
    ///
    /// \param  strText         ...
    /// \param  pbValue         ...
    wxOptionCheckBox( const wxString &strText, bool *pbValue );

  public:
    // Member functions
    virtual bool AddToSizer( wxBoxSizer *poSizer );     ///< ...
    virtual void Setup( wxWindow *poWindow, int nId );  ///< ...
    virtual void StoreValue();                          ///< Store the value in the connected variables.

  protected:
    // Member variables.
    bool *m_pbValue;                                    ///< ...
    wxCheckBox *m_poCheckBox;                           ///< ...
    wxString m_strText;                                 ///< ...
};


/////////////////////////////////////////////////////////////////////////////
//  class wxOptionTextBox
//
/// \brief The text input box for an options dialog.
///
/// This class represents a text (wxTextCtrl) option used in an
/// options dialog.
class wxOptionTextBox : public wxOptionBase
{
  public:
    /// \brief Default constructor.
    ///
    /// \param  strText         ...
    /// \param  pstrValue       ...
    /// \param  nWidth          ...
    wxOptionTextBox( const wxString &strText, wxString *pstrValue, int nWidth= 400 );

  public:
    // Member functions
    virtual bool AddToSizer( wxBoxSizer *poSizer );     ///< ...
    virtual void Setup( wxWindow *poWindow, int nId );  ///< ...
    virtual void StoreValue();                          ///< Store the value in the connected variables.

  protected:
    wxWindow *m_poWindow;                               ///< ...
    wxString m_strText;                                 ///< ...
    wxString *m_pstrValue;                              ///< ...
    wxTextCtrl *m_poTextCtrl;                           ///< ...
    int m_nWidth;                                       ///< ...
};


/////////////////////////////////////////////////////////////////////////////
//  class wxOptionComboBox
//
/// \brief The text input box with history for an options dialog.
///
/// This class represents a text option with history (wxComboBox)
/// used in an options dialog.
class wxOptionComboBox : public wxOptionBase
{
  public:
    /// \brief Default constructor.
    ///
    /// \param  strText         ...
    /// \param  pstrValue       ...
    /// \param  nWidth          ...
    wxOptionComboBox( const wxString &strText, wxString *pstrValue, int nWidth= 400 );

    /// \brief Virtual destructor.
    virtual ~wxOptionComboBox();

  public:
    // Member functions
    virtual bool AddToSizer( wxBoxSizer *poSizer );     ///< ...
    virtual void Setup( wxWindow *poWindow, int nId );  ///< ...
    virtual void StoreValue();                          ///< Store the value in the connected variables.

  protected:
    wxWindow *m_poWindow;                               ///< ...
    wxString m_strText;                                 ///< ...
    wxString *m_pstrValue;                              ///< ...
    wxComboBox *m_poComboBox;                           ///< ComboBox control
    int m_nWidth;                                       ///< ...
    int m_nId;                                          ///< ID of the control (for the config section).
    wxArrayString m_astrValueHistory;                   ///< Array of historical entries for the drop down list.
};


/////////////////////////////////////////////////////////////////////////////
//  class wxOptionsDialog
//
/// \brief The standard options dialog.
///
/// This class represents the application's options dialog.
class wxOptionsDialog : public wxDialog
{
  public:
    /// \brief Standard constructor
    ///
    /// \param poParentWindow   a pointer to parent window (the window owner).
    wxOptionsDialog( wxWindow *poParentWindow );

    /// Virtual destructor.
    virtual ~wxOptionsDialog();

  public:
    // Member functions
    virtual void SetupDialog();                             ///< Setup the dialog window.
    virtual bool AppendOption( wxOptionBase *poOption );    ///< Append the given option object to the options list.
    virtual wxWindow *GetParent(){ return this; }           ///< ...

  protected:
    void OnOk( wxCommandEvent& oEvent );                    ///< Handler for pressed OK button.

  protected:
    int m_nOptionCount;                                     ///< Number of option objects.
    int m_nOptionId;                                        ///< ID of the next control to add.
    wxOptionBase *m_apoOptions[WXOPTIONSDIALOGMAXOPTION];   ///< Array of option objects.

  private:
    // Any class wishing to process wxWindows events must use this macro.
    DECLARE_EVENT_TABLE()
};


#if __NOTYETRELEASED__ >= 2
/////////////////////////////////////////////////////////////////////////////
//  class wxOptionsBookDialog
//
/// \brief The list-/choice-/notebook options dialog.
///
/// This class represents the application's options dialog with tabs.
class wxOptionsBookDialog : public wxOptionsDialog
{
  public:
    /// \brief Standard constructor
    ///
    /// \param poParentWindow   a pointer to parent window (the window owner).
    /// \param strPageName      the name of the page/tab.
    /// \param poBookCtrl       a pointer to the note-/list-/choicebook control.
    wxOptionsBookDialog( wxWindow *poParentWindow, const wxString &strPageName,
                         wxBookCtrlBase *poBookCtrl= NULL );

  public:
    // Member functions
    virtual wxWindow *GetParent(){ return m_poParentForControls; } ///< ...
    virtual bool AddPage( const wxString &strPageName);     ///< ...

  protected:
    wxBookCtrlBase *m_poBookCtrl;                           ///< ...
    wxWindow *m_poParentForControls;                        ///< ...
};
#endif // __NOTYETRELEASED__

#endif // __WXOPTIONSDIALOG_H
